#!/usr/bin/ruby

require 'pp'
require 'gnuplot'
require 'pathname'
require "optparse"

USAGE = "Usage: #{File.basename("#{__FILE__}")} [options]"

## option analysis
options = {}
op = OptionParser.new
op.banner = USAGE
op.on("-e" , "--eps" , "Output eps format, alternative to png format"){options[:eps] = true}
op.parse!(ARGV)

dat_file = Pathname( __FILE__ ).sub_ext( ".dat").to_s
png_file = Pathname( __FILE__ ).sub_ext( ".png").to_s
eps_file = Pathname( __FILE__ ).sub_ext( ".eps").to_s

sizes = ['[-66:-45]', '[-65.2:-64]']

Gnuplot.open do |gp|
  Gnuplot::Plot.new(gp) do |plot|
    #pp  Pathname( __FILE__).sub_ext( ".eps").to_s
    #exit
    if options[:eps]
      plot.terminal "postscript eps enhanced color solid"
      plot.output   Pathname( __FILE__).sub_ext( ".eps").to_s
    else
      plot.terminal "png enhanced font 'IPA P ゴシック' fontscale 1.2"
      plot.output   Pathname( __FILE__).sub_ext( ".png").to_s
    end

    plot.title    "TODO"
    plot.xlabel   "TODO"
    plot.ylabel   "TODO"
    plot.xrange    sizes[0]
    plot.yrange    sizes[0]
    plot.nokey

    plot.data << Gnuplot::DataSet.new("x") do |ds|
      ds.with      = "lines"
      ds.linecolor = "rgb 'blue' linetype 1"
      ds.linewidth = 1
    end 

    lines = File.open(dat_file).readlines.select{|line| line !~ /^\s*\#/ }
    lines.map!{|i| i.strip.split(/  */).map{|i| i.to_f }}
    columns = lines.transpose
    1.upto(columns.size-1) do |i|
      plot.data << Gnuplot::DataSet.new([columns[0], columns[i]]) do |ds|
        ds.with      = "points"  # 点のみなら "points"
        ds.linecolor = "rgb 'red' linetype 1"
        ds.notitle
      end
    end

  end
end
